unit clipbrd_settable;

{$mode objfpc}{$H+}

interface

uses
  Classes, Dialogs, SysUtils, graphics, LCLType, ActiveX, strutils, mymetafile,
  clipbrd, clipbrd_csv, clipbrd_xmlspreadsheet, clipbrd_metafilepicture;

type
  TArrayTextTable = array of array of string;

const
  OBJECTDESCRIPTOR_FORMAT = {$IFDEF WINDOWS}'Object Descriptor'{$ELSE}'Object Descriptor'{$ENDIF};
  DATAOBJECT_FORMAT = {$IFDEF WINDOWS}'DataObject'{$ELSE}'DataObject'{$ENDIF};

var
  cfOBJECTDESCRIPTOR: TClipboardFormat = 0;
  cfDATAOBJECT: TClipboardFormat = 0;
{
 Для распознания данных в буфере обмена как таблицы, и вставки содержимого
 в AutoCAD как таблицы необходимо описать ниже указанную структуру.

 METAFILE PICTURE
 XML Spreadsheet   - Отвечает за форматирование текста в ячейках
 HTML Format
 UNICODE TEXT
 OEM TEXT
 TEXT
 CSV
 Object Description  - Отвечает за распознание данных буфера как таблицы Excel
 LOCALE
}

procedure ClipboardSetTable(ArrTable:TArrayTextTable);
procedure GetFileFromClpb(AFileName:String; AFormat:TClipboardFormat);

implementation

function CF_OBJECTDESCRIPTOR: TClipboardFormat;
begin
  if cfOBJECTDESCRIPTOR = 0 then
    cfOBJECTDESCRIPTOR := RegisterClipboardFormat(OBJECTDESCRIPTOR_FORMAT);
  Result := cfOBJECTDESCRIPTOR;
end;

function CF_DATAOBJECT: TClipboardFormat;
begin
  if cfDATAOBJECT = 0 then
    cfDATAOBJECT := RegisterClipboardFormat(DATAOBJECT_FORMAT);
  Result := cfDATAOBJECT;
end;

procedure SetFileToClpb(AFileName:String; AFormat:TClipboardFormat);
var
  FileStream        :TFileStream;
  sTmp              :string;
begin
  try
      sTmp:=(ExtractFilePath(Paramstr(0)))+'1\'+AFileName;
      FileStream := TFileStream.Create(sTmp,fmOpenRead or fmShareDenyNone);
      FileStream.Position:=0;
      ClipBoard.AddFormat(AFormat, FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure GetFileFromClpb(AFileName:String; AFormat:TClipboardFormat);
var
  Stream        :TMemoryStream;
  FileStream        :TFileStream;
  sTmp              :string;
begin
  try
      sTmp:=(ExtractFilePath(Paramstr(0)))+'1\'+AFileName;
      Stream := TMemoryStream.Create;
      FileStream := TFileStream.Create(sTmp,fmCreate);

      //CopyStreamFromClipboard(AFormat,FileStream);
      //Stream.Position:=0;
      //ClipBoard.GetFormat(AFormat, Stream);
      //ShowMessage(Inttostr(Stream.Size));
      //Stream.Position:=0;
      //FileStream.Write(Stream,Stream.Size);
      //FileStream.Position:=0;
  finally
    FileStream.Free;
    Stream.Free;
  end;
end;

procedure SetAsText(TextData: String);
var
  Stream: TStream;
begin
  if CF_TEXT = 0 then
    exit;

  Stream := TStringStream.Create;
  try

    if (TextData <> '') then
    begin
      Stream.Size := 0;
      Stream.Position := 0;
      Stream.WriteBuffer(Pointer(TextData)^, Length(TextData)+1); //Also write terminating zero
      Stream.Position := 0;
      ClipBoard.AddFormat(CF_TEXT, Stream);
    end;

  finally
    Stream.Free;
  end;
end;

function HexToStr(HexStr : String; const Text : boolean = True) : string;
var
  bHex : longint;
  sTemp : string;
  i : integer;
begin
  Result := '';
  if Length(HexStr) mod 2 <> 0 then
    Exit;//('Неправельная длина строки');

  for i := 2 to Length(HexStr) do
    if i mod 2 = 0 then
    begin
      sTemp := '$' + Copy(HexStr, i - 1, 2);
      bHex := hex2dec(sTemp);
      Result := Result + chr(bHex);
    end;
end;

procedure SetAsObjectDescription(TextData: String);
var
  Stream: TStream;
  r:tagOBJECTDESCRIPTOR;
begin
  if CF_OBJECTDESCRIPTOR = 0 then
    exit;
  {
  r.cbSize               :=0;
  r.clsid                :=sysutils.GUID_NULL;
  r.dwDrawAspect         :=0;
  r.sizel                :=TSize(0);
  r.pointl               :=0;
  r.dwStatus             :=0;
  r.dwFullUserTypeName   :=0;
  r.dwSrcOfCopy          :=0;
  }
  //Копия данных записанных Excel
  TextData:='800000003008020000000000C000000000000046010000000000000000000000000000000000000020000000340000005E0000001B0438044104420420004D006900630072006F0073006F0066007400200045007800630065006C0000001B043804410442043100210052003500430035003A00520031003100430037000000';
  TextData:=HexToStr(TextData);

  Stream := TStringStream.Create(TextData);
  try

      ClipBoard.AddFormat(CF_OBJECTDESCRIPTOR, Stream);

  finally
    Stream.Free;
  end;
end;

procedure ClipboardSetTable(ArrTable:TArrayTextTable);
var
  ic,ir, irCount, icCount : integer;
  sText           : String;
  sCSV            : String;
  sHTML           : String;
begin

  irCount           :=Length(ArrTable);
  if irCount=0 then exit;
  icCount           :=Length(ArrTable[0]);

  //Пишем CSV
  sCSV:='';
  for ir:=0 to irCount-1 do
  begin
    for ic:=0 to icCount-1 do
    begin
      sCSV:=sCSV+ArrTable[ir,ic];
      if ic<icCount-1 then
      begin
         sCSV:=sCSV+';';
      end;
    end;
    if ir<irCount-1 then
    begin
       sCSV:=sCSV+#13+#10;
    end;
  end;

  //Пишем TEXT; OEM TEXT; UNICODE TEXT
  sText:='';
  for ir:=0 to irCount-1 do
  begin
    for ic:=0 to icCount-1 do
    begin
      sText:=sText+ArrTable[ir,ic];
      if ic<icCount-1 then
      begin
         sText:=sText+#9;
      end;
    end;
    if ir<irCount-1 then
    begin
       sText:=sText+#13+#10;
    end;
  end;

  //Пишем HTML
  sHTML:='';
  sHTML:='<table border=0 cellpadding=0 cellspacing=0>';
  for ir:=0 to irCount-1 do
  begin
    sHTML:=sHTML+'<tr>';
    for ic:=0 to icCount-1 do
    begin
      sHTML:=sHTML+'<td>';
      sHTML:=sHTML+ArrTable[ir,ic];
      sHTML:=sHTML+'</td>';
    end;
    sHTML:=sHTML+'</tr>';
  end;
  sHTML:=sHTML+'</table>';
  //clipbrd.Clipboard.SetAsHtml(sHTML);

  {$IFDEF WINDOWS}
    SetFileToClpb('dataobject.dat',CF_DATAOBJECT);
    SetFileToClpb('metafilepicture.wmf',CF_METAFILEPICTURE);
    SetFileToClpb('xml.txt',CF_XMLSS);
    SetFileToClpb('html.txt',CF_HTML);
    SetFileToClpb('emetafilepicture.dat',CF_ENHANCEDMETAFILE);
    SetAsText(sText);
    SetAsCSV(sCSV);
    //SetAsMetafile(''); //обманка
    SetAsObjectDescription('');
  {$ELSE}

  {$ENDIF}

end;

end.

